@@ -28,11 +28,14 @@
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-theme.h>
 #include <goffice/utils/go-color.h>
+#include <goffice/utils/go-marker.h>
+#include <goffice/utils/go-math.h>
 #include <goffice/utils/go-units.h>
 
 #include <gsf/gsf-impl-utils.h>
-#include <gtk/gtknotebook.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
+
+#include <math.h>
 
 struct _GogLegend {
 	GogOutlinedObject base;
@@ -182,19 +185,19 @@ gog_legend_class_init (GogLegendClass *klass)
 		  NULL, NULL, NULL, NULL, NULL, NULL },
 	};
 
-	GObjectClass *gobject_klass   = (GObjectClass *) klass;
-	GogObjectClass *gog_klass = (GogObjectClass *) klass;
-	GogStyledObjectClass *style_klass = (GogStyledObjectClass *) klass;
+	GObjectClass         *gobject_klass = (GObjectClass *) klass;
+	GogObjectClass       *gog_klass     = (GogObjectClass *) klass;
+	GogStyledObjectClass *style_klass   = (GogStyledObjectClass *) klass;
 
 	parent_klass = g_type_class_peek_parent (klass);
 	
 	gobject_klass->set_property = gog_legend_set_property;
 	gobject_klass->get_property = gog_legend_get_property;
-	gog_klass->parent_changed = gog_legend_parent_changed;
-	gog_klass->update	  = gog_legend_update;
-	gog_klass->populate_editor	= gog_legend_populate_editor;
-	gog_klass->view_type	  = gog_legend_view_get_type ();
-	style_klass->init_style	  = gog_legend_init_style;
+	gog_klass->parent_changed   = gog_legend_parent_changed;
+	gog_klass->update           = gog_legend_update;
+	gog_klass->populate_editor  = gog_legend_populate_editor;
+	gog_klass->view_type        = gog_legend_view_get_type ();
+	style_klass->init_style     = gog_legend_init_style;
 	
 	gog_object_register_roles (gog_klass, roles, G_N_ELEMENTS (roles));
 
@@ -222,9 +225,18 @@ GSF_CLASS (GogLegend, gog_legend,
 
 typedef struct {
 	GogOutlinedView	base;
-	double		line_height;
+	gboolean	is_vertical;
+	double		element_width;
+	double		element_height;
+	unsigned	element_per_blocks;
+	double 		swatch_w;
+	double 		swatch_h;
+	unsigned	num_blocks;
 	gboolean	uses_lines;
+	double		label_offset;
+	double		font_size;
 } GogLegendView;
+
 typedef GogOutlinedViewClass	GogLegendViewClass;
 
 #define GOG_LEGEND_VIEW_TYPE	(gog_legend_view_get_type ())
@@ -233,152 +245,240 @@ typedef GogOutlinedViewClass	GogLegendViewClass;
 
 static GogViewClass *lview_parent_klass;
 
-typedef struct {
-	GogView const *view;
-	GogViewRequisition maximum;
-	gboolean	uses_lines;
-	GogStyle* legend_style;
-} size_closure;
-
 static void
-cb_size_elements (unsigned i, GogStyle const *style, char const *name,
-		  size_closure *dat)
+cb_size_elements (unsigned i, GogStyle const *style, 
+		  char const *name, GogLegendView *glv)
 {
+	GogView *view = GOG_VIEW (glv);
 	GOGeometryAABR aabr;
 	
-	gog_renderer_push_style (dat->view->renderer, dat->legend_style);
-	gog_renderer_get_text_AABR (dat->view->renderer, name, &aabr);
-	gog_renderer_pop_style (dat->view->renderer);
+	gog_renderer_get_text_AABR (view->renderer, name, &aabr);
 
-	if (dat->maximum.w < aabr.w)
-		dat->maximum.w = aabr.w;
-	if (dat->maximum.h < aabr.h)
-		dat->maximum.h = aabr.h;
-	if (!dat->uses_lines && (style->interesting_fields & GOG_STYLE_LINE))
-		dat->uses_lines = TRUE;
+	if (glv->element_width < aabr.w)
+		glv->element_width = aabr.w;
+	if (glv->element_height < aabr.h)
+		glv->element_height = aabr.h;
+	if (!glv->uses_lines && (style->interesting_fields & GOG_STYLE_LINE))
+		glv->uses_lines = TRUE;
 }
 
 static void
-gog_legend_view_size_request (GogView *v, GogViewRequisition *avail)
+gog_legend_view_size_request (GogView *v, 
+			      GogViewRequisition const *available, 
+			      GogViewRequisition *req)
 {
-	size_closure dat;
-	GogViewRequisition res;
 	GogChart *chart = GOG_CHART (v->model->parent);
+	GogLegendView *glv = GOG_LEGEND_VIEW (v);
 	GogLegend *l = GOG_LEGEND (v->model);
-	unsigned n, mult = 1;
+	GogViewRequisition child_req, residual;
+	GogStyle *style;
+	double available_space, element_size;
+	unsigned num_elements;
+
+	residual = *available;
+	req->w = req->h = 0;
+	gog_view_size_child_request (v, available, req, &child_req);
+	lview_parent_klass->size_request (v, available, req);
+	residual.w -= req->w;
+	residual.h -= req->h;
+
+	glv->is_vertical = gog_object_get_position_flags (GOG_OBJECT (l), GOG_POSITION_COMPASS) &
+		(GOG_POSITION_E | GOG_POSITION_W);
+
+	gog_chart_get_cardinality (chart, NULL, &num_elements);
+	
+	style = gog_styled_object_get_style (GOG_STYLED_OBJECT (l));
+	gog_renderer_push_style (v->renderer, style);
+	
+	glv->font_size = pango_font_description_get_size (style->font.font->desc) / PANGO_SCALE;
+	glv->swatch_w = gog_renderer_pt2r_x (v->renderer, glv->font_size);
+	glv->swatch_h = gog_renderer_pt2r_y (v->renderer, glv->font_size);
+					     
+	glv->element_width = 0.;
+	glv->element_height = 1.2 * glv->swatch_h;
+	glv->uses_lines = FALSE;
 
-#warning TODO : make this smarter (multiple columns and shrinking text)
-	dat.view = v;
-	dat.maximum.w = 0.;
-	dat.maximum.h = gog_renderer_pt2r_y (v->renderer, l->swatch_size_pts);
-	dat.uses_lines = FALSE;
-	dat.legend_style = GOG_STYLED_OBJECT (l)->style;
 	gog_chart_foreach_elem (chart, TRUE,
-		(GogEnumFunc) cb_size_elements, &dat);
-	((GogLegendView *)v)->line_height = dat.maximum.h;
-	((GogLegendView *)v)->uses_lines = dat.uses_lines;
+		(GogEnumFunc) cb_size_elements, glv);
 
-	if (dat.uses_lines)
-		mult = 3;
+	gog_renderer_pop_style (v->renderer);
 
-	/* 1/2 between swatch and label */
-	res.w = dat.maximum.w + gog_renderer_pt2r_x (v->renderer,
-		mult * l->swatch_size_pts + .5 * l->swatch_padding_pts);
-	gog_chart_get_cardinality (chart, NULL, &n);
-	res.h = n * dat.maximum.h;
+	glv->label_offset = (glv->uses_lines ? 3.5 : 1.5) * glv->swatch_w;
+	glv->element_width += glv->label_offset + glv->swatch_w;
 
-	gog_view_size_child_request (v, avail, &res);
-	avail->w = res.w;
-	avail->h = res.h;
-	lview_parent_klass->size_request (v, avail);
+	available_space = glv->is_vertical ? residual.h : residual.w;
+	element_size = glv->is_vertical ? glv->element_height : glv->element_width;	
+	
+	glv->element_per_blocks = available_space > 0. ? floor (available_space / element_size) : 0;
+	
+	if (glv->element_per_blocks < 1) {
+	       	req->w = req->h = -1;
+		return;
+	}
+
+	glv->num_blocks = floor ((num_elements - 1) / glv->element_per_blocks) + 1;
+
+	if (glv->is_vertical) {
+		req->h += MIN (glv->element_per_blocks, num_elements) * glv->element_height;
+		req->w += glv->num_blocks * glv->element_width - glv->swatch_w;
+	} else {
+		req->h += glv->num_blocks * glv->element_height;
+		req->w += MIN (glv->element_per_blocks, num_elements) * glv->element_width - glv->swatch_w;
+	}
+
+	req->w = MAX (child_req.w, req->w);
+	req->h = MAX (child_req.h, req->h);
+}
+
+typedef struct {
+	double size_max;
+	double size_min;
+} SwatchScaleClosure;
+
+static void
+cb_swatch_scale (unsigned i, GogStyle const *style, char const *name,
+		 SwatchScaleClosure *data)
+{
+	double size = go_marker_get_size (style->marker.mark);
+	if (data->size_max < size)
+		data->size_max = size;
+	if (data->size_min > size)
+		data->size_min = size;
 }
 
 typedef struct {
 	GogView const *view;
+	double x, y;
+	double element_step_x, element_step_y;
+	double block_step_x, block_step_y;
 	GogViewAllocation swatch;
-	double step;
-	double label_offset;
-	double bottom;
 	ArtVpath line_path[3];
-} render_closure;
+	double swatch_scale_a, swatch_scale_b;
+} RenderClosure;
 
 static void
 cb_render_elements (unsigned i, GogStyle const *base_style, char const *name,
-		    render_closure *dat)
+		    RenderClosure *data)
 {
-	GogViewAllocation swatch = dat->swatch;
-	GogView  const   *v = dat->view;
+	GogView const *view = data->view;
+	GogLegendView *glv = GOG_LEGEND_VIEW (view);
+	GogRenderer *renderer = view->renderer;
 	GogStyle *style = NULL;
 	GogViewAllocation pos, rectangle;
-	GogStyledObject *obj = GOG_STYLED_OBJECT (dat->view->model);
-	GogStyle *legend_style = obj->style;
-	
-	swatch.y += i * dat->step;
-	/* Allow for floating point inaccuracy */
-	if (swatch.y > dat->bottom + 0.0001)
-		return;
 
+	if (i > 0) {
+		if ((i % glv->element_per_blocks) != 0) {
+			data->x += data->element_step_x;
+			data->y += data->element_step_y;
+		} else {
+			data->x += data->block_step_x;
+			data->y += data->block_step_y;
+		}
+	}
+	
 	if (base_style->interesting_fields & GOG_STYLE_LINE) { /* line and marker */
-		style = (GogStyle *)base_style;
-		gog_renderer_push_style (v->renderer, style);
-		dat->line_path[0].y = dat->line_path[1].y =  swatch.y + dat->step / 2.;
-		gog_renderer_draw_sharp_path (v->renderer, dat->line_path);
-		gog_renderer_draw_marker (v->renderer,
-			(dat->line_path[0].x + dat->line_path[1].x) / 2.,
-			dat->line_path[0].y);
+		style = gog_style_dup (base_style);
+		go_marker_set_size (style->marker.mark, 
+				    go_marker_get_size (style->marker.mark) *
+				    data->swatch_scale_a + data->swatch_scale_b);
+
+		gog_renderer_push_style (renderer, style);
+		data->line_path[0].x = data->x;
+		data->line_path[1].x = data->x + data->swatch.w * 3.;
+		data->line_path[0].y = 
+			data->line_path[1].y = data->y + glv->element_height / 2.;
+		gog_renderer_draw_sharp_path (renderer, data->line_path);
+		gog_renderer_draw_marker (renderer, data->x + data->swatch.w  * 1.5, 
+					  data->line_path[0].y);
 	} else {					/* area swatch */
 		style = gog_style_dup (base_style);
-		style->outline.width = 0; /* hairline */
+		style->outline.width = 0; 
 		style->outline.color = RGBA_BLACK;
 
-		rectangle = swatch;
-		rectangle.y += (dat->step - swatch.h) / 2.0;
+		rectangle = data->swatch;
+		rectangle.x += data->x;
+		rectangle.y += data->y;
 
-		gog_renderer_push_style (v->renderer, style);
-		gog_renderer_draw_sharp_rectangle (v->renderer, &rectangle);
+		gog_renderer_push_style (renderer, style);
+		gog_renderer_draw_sharp_rectangle (renderer, &rectangle);
 	}
-	pos.x = swatch.x + dat->label_offset;
-	pos.y = swatch.y;
-	pos.h = pos.w = -1;
-	gog_renderer_pop_style (v->renderer);
-	gog_renderer_push_style (v->renderer, legend_style);
-	gog_renderer_draw_text (v->renderer, name, &pos, GTK_ANCHOR_NW, NULL);
+	gog_renderer_pop_style (renderer);
 
-	gog_renderer_pop_style (v->renderer);
+	pos.x = data->x + glv->label_offset;
+	pos.y = data->y + glv->element_height / 2.0;
+	pos.w = pos.h = -1;
+	gog_renderer_draw_text (renderer, name, &pos, GTK_ANCHOR_W, NULL);
 
-	if (style != base_style)
+	if (style != base_style && style != NULL)
 		g_object_unref (style);
 }
 
 static void
 gog_legend_view_render (GogView *v, GogViewAllocation const *bbox)
 {
-	render_closure dat;
+	GogLegendView *glv = GOG_LEGEND_VIEW (v);
 	GogLegend *l = GOG_LEGEND (v->model);
-	double pad_x = gog_renderer_pt2r_x (v->renderer, l->swatch_padding_pts);
+	GogStyle *style = gog_styled_object_get_style (GOG_STYLED_OBJECT (l));
+	RenderClosure data;
+	SwatchScaleClosure swatch_data;
+	double swatch_size_min;
+	double swatch_size_max;
 
 	(lview_parent_klass->render) (v, bbox);
-
-	dat.view = v;
-	dat.swatch.x  = v->residual.x;
-	dat.swatch.y  = v->residual.y;
-	dat.swatch.w  = gog_renderer_pt2r_x (v->renderer, l->swatch_size_pts);
-	dat.swatch.h  = gog_renderer_pt2r_y (v->renderer, l->swatch_size_pts);
-	dat.label_offset = dat.swatch.w + pad_x / 2.;
-	if (((GogLegendView *)v)->uses_lines) {
-		dat.line_path[0].code = ART_MOVETO;
-		dat.line_path[1].code = ART_LINETO;
-		dat.line_path[2].code = ART_END;
-		dat.line_path[0].x = dat.swatch.x;
-		dat.line_path[1].x = dat.swatch.x + 3. * dat.swatch.w;
-		dat.swatch.x += dat.swatch.w;
-		dat.label_offset += dat.swatch.w;
-	}
-	dat.step      = ((GogLegendView *)v)->line_height;
-	dat.bottom    = v->residual.y + v->residual.h -
-		((GogLegendView *)v)->line_height;
+	
+	if (glv->element_per_blocks < 1)
+		return;
+	
+	gog_renderer_push_style (v->renderer, style);
+	
+	swatch_data.size_max = 0.0;
+	swatch_data.size_min = glv->font_size;
+	
 	gog_chart_foreach_elem (GOG_CHART (v->model->parent), TRUE,
-		(GogEnumFunc) cb_render_elements, &dat);
+		(GogEnumFunc) cb_swatch_scale, &swatch_data);
+
+	swatch_size_max = glv->font_size;
+	swatch_size_min = glv->font_size / 4.0;
+	if (swatch_data.size_max < swatch_size_max)
+		swatch_data.size_max = swatch_size_max;
+	if (swatch_data.size_min > swatch_size_min)
+		swatch_data.size_min = swatch_size_min;
+	if (go_sub_epsilon (fabs (swatch_data.size_max - swatch_data.size_min)) > 0.0) {
+		data.swatch_scale_a = (swatch_size_max - swatch_size_min) / 
+			(swatch_data.size_max - swatch_data.size_min);
+		data.swatch_scale_b = - swatch_data.size_min * 
+			data.swatch_scale_a +swatch_size_min;
+	} else {
+		data.swatch_scale_a = 1.0;
+		data.swatch_scale_b = 0.0;
+	}
+
+	if (glv->uses_lines) {
+		data.line_path[0].code = ART_MOVETO;
+		data.line_path[1].code = ART_LINETO;
+		data.line_path[2].code = ART_END;
+	}
+	
+	data.view = v;
+	data.x = v->residual.x;
+	data.y = v->residual.y;
+	data.element_step_x = glv->is_vertical ? 0 : glv->element_width;
+	data.element_step_y = glv->is_vertical ? glv->element_height : 0;
+	data.block_step_x = glv->is_vertical ? 
+		+ glv->element_width : 
+		- glv->element_width * (glv->element_per_blocks - 1);
+	data.block_step_y = glv->is_vertical ? 
+		- glv->element_height * (glv->element_per_blocks - 1) :
+		+ glv->element_height;
+	data.swatch.w = glv->swatch_w;
+	data.swatch.h = glv->swatch_h;
+	data.swatch.x = (glv->label_offset - 1.5 * data.swatch.w) * .5;
+	data.swatch.y = (glv->element_height - data.swatch.h) * .5;
+	
+	gog_chart_foreach_elem (GOG_CHART (v->model->parent), TRUE,
+		(GogEnumFunc) cb_render_elements, &data);
+
+	gog_renderer_pop_style (v->renderer);
 }
 
 static void
