@@ -111,9 +111,6 @@ gog_editor_get_notebook (GogEditor *editor)
 		page_count = 1;
 	}
 
-	if (page_count == 1)
-		gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
-
 	if (editor->store_page != NULL) {
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), *editor->store_page);
 		g_signal_connect (G_OBJECT (notebook),
@@ -143,9 +140,9 @@ typedef struct {
 
 static GogPositionFlagDesc const position_compass[] = {
 	{N_("Top"), 		"top",		GOG_POSITION_N},
-	{N_("Top right"), 	"top-righ",	GOG_POSITION_N|GOG_POSITION_E},
+	{N_("Top right"), 	"top-right",	GOG_POSITION_N|GOG_POSITION_E},
 	{N_("Right"), 		"right",	GOG_POSITION_E},
-	{N_("Bottom right"), 	"bottom-righ",	GOG_POSITION_E|GOG_POSITION_S},
+	{N_("Bottom right"), 	"bottom-right",	GOG_POSITION_E|GOG_POSITION_S},
 	{N_("Bottom"),		"bottom",	GOG_POSITION_S},
 	{N_("Bottom left"),	"bottom-left",	GOG_POSITION_S|GOG_POSITION_W},
 	{N_("Left"),		"left",		GOG_POSITION_W},
