@@ -27,10 +27,12 @@
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-error-bar.h>
 #include <goffice/data/go-data.h>
-#include <goffice/gtk/goffice-gtk.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
+
+#ifdef GOFFICE_WITH_GTK
+#include <goffice/gtk/goffice-gtk.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkhseparator.h>
@@ -39,6 +41,7 @@
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtkcheckbutton.h>
+#endif
 
 #include <string.h>
 
@@ -104,6 +107,7 @@ gog_series_element_get_property (GObject *obj, guint param_id,
 	}
 }
 
+#ifdef GOFFICE_WITH_GTK
 static void
 cb_index_changed (GtkSpinButton *spin_button, GogSeriesElement *element)
 {
@@ -162,6 +166,7 @@ gog_series_element_populate_editor (GogObject *gobj,
 
 	gog_editor_set_store_page (editor, &series_element_pref_page);
 }
+#endif
 
 static void
 gog_series_element_init_style (GogStyledObject *gso, GogStyle *style)
@@ -188,7 +193,9 @@ gog_series_element_class_init (GogSeriesElementClass *klass)
 	gobject_klass->set_property = gog_series_element_set_property;
 	gobject_klass->get_property = gog_series_element_get_property;
 
+#ifdef GOFFICE_WITH_GTK
 	gog_klass->populate_editor 	= gog_series_element_populate_editor;
+#endif
 	style_klass->init_style	    	= gog_series_element_init_style;
 
 	gog_klass->use_parent_as_proxy  = TRUE;
@@ -210,7 +217,7 @@ static gboolean
 regression_curve_can_add (GogObject const *parent)
 {
 	GogSeries *series = GOG_SERIES (parent);
-	return (series->acceptable_children & GOG_SERIES_ACCEPT_REGRESSION_CURVE) != 0;
+	return (series->acceptable_children & GOG_SERIES_ACCEPT_TREND_LINE) != 0;
 }
 
 static void
@@ -330,6 +337,7 @@ gog_series_get_property (GObject *obj, guint param_id,
 	}
 }
 
+#ifdef GOFFICE_WITH_GTK
 static unsigned
 make_dim_editor (GtkTable *table, unsigned row, GtkWidget *editor,
 		 char const *name, GogSeriesPriority priority, gboolean is_shared)
@@ -440,6 +448,7 @@ gog_series_populate_editor (GogObject *gobj,
 
 	gog_editor_set_store_page (editor, &series_pref_page);
 }
+#endif
 
 static void
 gog_series_update (GogObject *obj)
@@ -475,6 +484,14 @@ gog_series_class_init (GogSeriesClass *klass)
 		  regression_curve_post_add,
 		  regression_curve_pre_remove,
 		  NULL },
+		{ N_("Trend line"), "GogTrendLine",	2,
+		  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_TYPE,
+		  regression_curve_can_add,
+		  NULL,
+		  NULL,
+		  regression_curve_post_add,
+		  regression_curve_pre_remove,
+		  NULL },
 	};
 
 	GObjectClass *gobject_klass = (GObjectClass *) klass;
@@ -486,7 +503,9 @@ gog_series_class_init (GogSeriesClass *klass)
 	gobject_klass->set_property	= gog_series_set_property;
 	gobject_klass->get_property	= gog_series_get_property;
 
+#ifdef GOFFICE_WITH_GTK
 	gog_klass->populate_editor	= gog_series_populate_editor;
+#endif
 	gog_klass->update		= gog_series_update;
 	style_klass->init_style 	= gog_series_init_style;
 	/* series do not have views, so just forward signals from the plot */
@@ -540,7 +559,7 @@ gog_series_dataset_set_dim (GogDataset *set, int dim_i,
 	GogSeries *series = GOG_SERIES (set);
 	GogGraph *graph = gog_object_get_graph (GOG_OBJECT (series));
 
-	g_return_if_fail (GOG_PLOT (series->plot) != NULL);
+	g_return_if_fail (IS_GOG_PLOT (series->plot));
 
 	if (dim_i < 0) {
 		char *name = NULL;
@@ -624,7 +643,7 @@ GSF_CLASS_FULL (GogSeries, gog_series,
 GogPlot *
 gog_series_get_plot (GogSeries const *series)
 {
-	g_return_val_if_fail (GOG_SERIES (series) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_SERIES (series), NULL);
 	return series->plot;
 }
 
@@ -637,7 +656,7 @@ gog_series_get_plot (GogSeries const *series)
 gboolean
 gog_series_is_valid (GogSeries const *series)
 {
-	g_return_val_if_fail (GOG_SERIES (series) != NULL, FALSE);
+	g_return_val_if_fail (IS_GOG_SERIES (series), FALSE);
 	return series->is_valid;
 }
 
@@ -655,8 +674,8 @@ gog_series_check_validity (GogSeries *series)
 	unsigned i;
 	GogSeriesDesc const *desc;
 
-	g_return_if_fail (GOG_SERIES (series) != NULL);
-	g_return_if_fail (GOG_PLOT (series->plot) != NULL);
+	g_return_if_fail (IS_GOG_SERIES (series));
+	g_return_if_fail (IS_GOG_PLOT (series->plot));
 
 	desc = &series->plot->desc.series;
 	for (i = series->plot->desc.series.num_dim; i-- > 0; )
@@ -677,17 +696,17 @@ gog_series_check_validity (GogSeries *series)
 gboolean
 gog_series_has_legend (GogSeries const *series)
 {
-	g_return_val_if_fail (GOG_SERIES (series) != NULL, FALSE);
+	g_return_val_if_fail (IS_GOG_SERIES (series), FALSE);
 	return series->has_legend;
 }
 
 /**
  * gog_series_set_index :
- * @series : #GogSeries
- * @index :
- * @is_manual :
+ * @series: #GogSeries
+ * @ind:
+ * @is_manual:
  *
- * if @index >= 0 attempt to assign the new index.  Auto
+ * If @index >= 0 attempt to assign the new index.  Auto
  * indicies (@is_manual == FALSE) will not override the current
  * index if it is manual.  An @index < 0, will reset the index to
  * automatic and potentially queue a revaluation of the parent
@@ -696,7 +715,7 @@ gog_series_has_legend (GogSeries const *series)
 void
 gog_series_set_index (GogSeries *series, int ind, gboolean is_manual)
 {
-	g_return_if_fail (GOG_SERIES (series) != NULL);
+	g_return_if_fail (IS_GOG_SERIES (series));
 
 	if (ind < 0) {
 		if (series->manual_index && series->plot != NULL)
@@ -716,27 +735,29 @@ gog_series_set_index (GogSeries *series, int ind, gboolean is_manual)
 }
 
 /**
- * gog_series_get_name :
- * @series : #GogSeries
+ * gog_series_get_name:
+ * @series : a #GogSeries
  *
- * Returns the _src_ of the name associated with the series
- * NOTE : this is _NOT_ the actual name
- * no references are added on the result.
+ * Gets the _source_ of the name associated with the series.
+ * NOTE : this is _NOT_ the actual name.
+ *
+ * return value: a #GODataScalar, without added reference.
  **/
 GODataScalar *
 gog_series_get_name (GogSeries const *series)
 {
-	g_return_val_if_fail (GOG_SERIES (series) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_SERIES (series), NULL);
 	return GO_DATA_SCALAR (series->values[-1].data);
 }
 
 /**
  * gog_series_set_name :
- * @series : #GogSeries
- * @name_src : #GOData
- * @err : #Gerror
+ * @series : a #GogSeries
+ * @name_src : a #GODataScalar
+ * @err : a #Gerror
+ *
+ * Absorbs a ref to @name_src.
  *
- * Absorbs a ref to @name_src
  **/
 void
 gog_series_set_name (GogSeries *series, GODataScalar *name_src, GError **err)
@@ -784,7 +805,7 @@ gog_series_get_valid_element_index (GogSeries const *series, int old_index, int
 	int index;
 	GList *ptr;
 
-	g_return_val_if_fail (GOG_SERIES (series) != NULL, -1);
+	g_return_val_if_fail (IS_GOG_SERIES (series), -1);
 
 	if ((desired_index >= (int) series->num_elements) ||
 	    (desired_index < 0))
@@ -820,7 +841,7 @@ gog_series_get_element (GogSeries const *series, int index)
 	GList *ptr;
 	GogSeriesElement *element;
 
-	g_return_val_if_fail (GOG_SERIES (series) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_SERIES (series), NULL);
 
 	for (ptr = series->overrides; ptr != NULL; ptr = ptr->next) {
 		element = GOG_SERIES_ELEMENT (ptr->data);
@@ -830,3 +851,29 @@ gog_series_get_element (GogSeries const *series, int index)
 	
 	return NULL;
 }
+
+unsigned
+gog_series_get_xy_data (GogSeries const *series,
+					double const **x, double const **y)
+{
+	GogSeriesClass	*klass = GOG_SERIES_GET_CLASS (series);
+	int first, last, res, nb;
+	g_return_val_if_fail (gog_series_is_valid (GOG_SERIES (series)), 0);
+	gog_dataset_dims (GOG_DATASET (series), &first, &last);
+	g_return_val_if_fail (last >= 1, 0);
+	if (klass->get_xy_data != NULL)
+		return (klass->get_xy_data) (series, x, y);
+	*y = go_data_vector_get_values (
+		GO_DATA_VECTOR (series->values[1].data));
+	res = go_data_vector_get_len (
+		GO_DATA_VECTOR (series->values[1].data));
+	if (series->values[0].data) {
+		*x = go_data_vector_get_values (
+			GO_DATA_VECTOR (series->values[0].data));
+		nb = go_data_vector_get_len (
+			GO_DATA_VECTOR (series->values[0].data));
+		if (res > nb)
+			res = nb;
+	}
+	return res;
+}
