@@ -75,16 +75,18 @@ gog_pie_series_element_get_property (GObject *obj, guint param_id,
 	}
 }
 
+#ifdef WITH_GTK
 extern gpointer gog_pie_series_element_pref (GogPieSeriesElement *element, GOCmdContext *cc);
 static gpointer
 gog_pie_series_element_populate_editor (GogObject *gobj,
 					GogEditor *editor,
-			       GOCmdContext *cc)
+					GOCmdContext *cc)
 {
 	GtkWidget *widget = gog_pie_series_element_pref (GOG_PIE_SERIES_ELEMENT (gobj), cc);
 	gog_editor_add_page (editor, widget, _("Settings"));
 	return widget;
 }
+#endif
 
 static void
 gog_pie_series_element_class_init (GogPieSeriesElementClass *klass)
@@ -95,7 +97,9 @@ gog_pie_series_element_class_init (GogPieSeriesElementClass *klass)
 	gobject_klass->get_property = gog_pie_series_element_get_property;
 	
 	ppe_parent_klass 	= g_type_class_peek_parent (klass);
+#ifdef WITH_GTK
 	klass->gse_populate_editor	= gog_pie_series_element_populate_editor;
+#endif
 
 	g_object_class_install_property (gobject_klass, ELEMENT_SEPARATION,
 		g_param_spec_float ("separation", "separation",
@@ -118,7 +122,8 @@ enum {
 	PLOT_PROP_0,
 	PLOT_PROP_INITIAL_ANGLE,
 	PLOT_PROP_DEFAULT_SEPARATION,
-	PLOT_PROP_IN_3D
+	PLOT_PROP_IN_3D,
+	PLOT_PROP_SPAN
 };
 
 GOFFICE_PLUGIN_MODULE_HEADER;
@@ -139,11 +144,14 @@ gog_pie_plot_set_property (GObject *obj, guint param_id,
 		break;
 	case PLOT_PROP_DEFAULT_SEPARATION :
 		f = g_value_get_float (value);
-		pie->default_separation = MIN (f, 5.);
+		pie->default_separation = CLAMP (f, 0., 5.);
 		break;
 	case PLOT_PROP_IN_3D :
 		pie->in_3d = g_value_get_boolean (value);
 		break;
+	case PLOT_PROP_SPAN :
+		pie->span = g_value_get_float (value);
+		break;
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 return; /* NOTE : RETURN */
@@ -170,6 +178,9 @@ gog_pie_plot_get_property (GObject *obj, guint param_id,
 	case PLOT_PROP_IN_3D :
 		g_value_set_boolean (value, pie->in_3d);
 		break;
+	case PLOT_PROP_SPAN :
+		g_value_set_float (value, pie->span);
+		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
 	}
@@ -185,12 +196,14 @@ extern gpointer gog_pie_plot_pref (GogPiePlot *pie, GOCmdContext *cc);
 static void
 gog_pie_plot_populate_editor (GogObject *item, 
 			      GogEditor *editor,
-		    G_GNUC_UNUSED GogDataAllocator *dalloc,
-		    GOCmdContext *cc)
+			      G_GNUC_UNUSED GogDataAllocator *dalloc,
+			      GOCmdContext *cc)
 {
 	gog_editor_add_page (editor, 
 			     gog_pie_plot_pref (GOG_PIE_PLOT (item), cc),
 			     _("Properties"));
+
+	(GOG_OBJECT_CLASS(pie_parent_klass)->populate_editor) (item, editor, dalloc, cc);
 }
 
 static void
@@ -229,6 +242,11 @@ gog_pie_plot_class_init (GogPlotClass *plot_klass)
 			"Draw 3d wedges",
 			FALSE,
 			G_PARAM_READWRITE));
+	g_object_class_install_property (gobject_klass, PLOT_PROP_SPAN,
+		g_param_spec_float ("span", _("span"),
+			_("Total angle used as a percentage of the full circle"),
+			10., 100., 100.,
+			G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 
 	{
 		static GogSeriesDimDesc dimensions[] = {
@@ -250,6 +268,7 @@ static void
 gog_pie_plot_init (GogPiePlot *pie)
 {
 	pie->base.vary_style_by_element = TRUE;
+	pie->span = 100.;
 }
 
 GSF_DYNAMIC_CLASS (GogPiePlot, gog_pie_plot,
@@ -355,9 +374,166 @@ GSF_DYNAMIC_CLASS (GogRingPlot, gog_ring_plot,
 	GOG_PIE_PLOT_TYPE)
 
 /*****************************************************************************/
+
+#ifdef WITH_GTK
+typedef struct {
+	double x, y, r;
+	double start_pos;
+       	double start_distance;
+} MovePieData;
+
+static gboolean
+find_element (GogView *view, double cx, double cy, double x, double y, 
+	      unsigned int *index, GogPieSeries **series)
+{
+	GogPiePlot *pie = GOG_PIE_PLOT (view->model);
+	GSList *ptr;
+	double *vals, scale, len = 0, theta;
+
+	*series = NULL;
+	*index = 0;
+
+	for (ptr = pie->base.series ; ptr != NULL ; ptr = ptr->next)
+		if (gog_series_is_valid (GOG_SERIES (*series = ptr->data)))
+			break;
+	if (ptr == NULL)
+		return FALSE;
+
+	theta = (atan2 (y - cy, x - cx) 
+		 * 180 / M_PI - pie->initial_angle + 90.) / pie->span / 3.6;
+	if (theta < 0)
+		theta += 1.;
+
+	vals = go_data_vector_get_values (GO_DATA_VECTOR ((*series)->base.values[1].data));
+	scale = 1 / (*series)->total;
+	for (*index = 0 ; *index < (*series)->base.num_elements; (*index)++) {
+		len = fabs (vals[*index]) * scale;
+		if (go_finite (len) && len > 1e-3) {
+			theta -= len;
+			if (theta < 0)
+				break;
+		}
+	}
+	return TRUE;
+}
+
+static gboolean
+gog_tool_move_pie_point (GogView *view, double x, double y, GogObject **gobj)
+{
+	GogPieSeries *series;
+	double r = view->allocation.h, cx, cy;
+	unsigned int index;
+	
+	if (r > view->allocation.w)
+		r = view->allocation.w;
+	r /= 2.;
+	cx = view->allocation.x + view->allocation.w/2.;
+	cy = view->allocation.y + view->allocation.h/2.;
+
+	if (hypot (x - cx, y - cy) > fabs (r))
+		return FALSE;
+	
+	if (find_element (view, cx, cy, x, y, &index, &series)) 
+		*gobj = GOG_OBJECT (gog_series_get_element (GOG_SERIES (series), index));
+
+	return TRUE;
+}
+
+static void
+gog_tool_move_pie_render (GogView *view)
+{
+	GogViewAllocation rectangle;
+	ArtVpath *path;
+	double r = view->allocation.h;
+	
+	if (r > view->allocation.w)
+		r = view->allocation.w;
+
+	rectangle.x = view->allocation.x + (view->allocation.w - r) / 2.0;
+	rectangle.y = view->allocation.y + (view->allocation.h - r) / 2.0;
+	rectangle.w = r;
+	rectangle.h = r;
+
+	path = gog_renderer_get_rectangle_vpath (&rectangle);
+	gog_renderer_draw_sharp_path (view->renderer, path);
+	art_free (path);
+}
+
+static void
+gog_tool_move_pie_init (GogToolAction *action)
+{
+	GogPiePlot *pie = GOG_PIE_PLOT (action->view->model);
+	MovePieData *data = g_new0 (MovePieData, 1);
+	GogViewAllocation area = action->view->allocation;
+	
+	data->r = area.h;
+	if (data->r > area.w)
+		data->r = area.w;
+	data->r /= 2.;
+	data->start_pos = (data->r * (0.5 + pie->default_separation)) / (1 + pie->default_separation);
+	data->x = area.x + area.w / 2.;
+	data->y = area.y + area.h / 2.;
+	data->start_distance = hypot (action->start_x - data->x, action->start_y - data->y);
+
+	action->data = data;
+}
+
+static void
+gog_tool_move_pie_move (GogToolAction *action, double x, double y)
+{
+	GogPiePlot *pie = GOG_PIE_PLOT (action->view->model);
+	MovePieData *data = action->data;
+	double pos, separation;
+
+	pos = data->start_pos - 
+		((x - action->start_x) * (data->x - action->start_x) +
+		 (y - action->start_y) * (data->y - action->start_y)) / 
+		data->start_distance; 
+	separation = (pos - 0.5 * data->r) / (data->r - pos);
+					     
+	gog_pie_plot_set_default_separation (pie, separation);
+}
+
+static void
+gog_tool_move_pie_double_click (GogToolAction *action)
+{
+	MovePieData *data = action->data;
+	GogPieSeries *series;
+	GogObject *obj;
+	unsigned int index;
+
+	if (!find_element (action->view, data->x, data->y, 
+			   action->start_x, action->start_y,
+			   &index, &series))
+		return;
+	
+	obj = GOG_OBJECT (gog_series_get_element (GOG_SERIES (series), index));
+	if (obj == NULL) {
+		obj = g_object_new (gog_pie_series_element_get_type (),
+				    "index", index, NULL);
+		gog_object_add_by_name (GOG_OBJECT (series), "Point", obj);
+	}
+}
+
+static GogTool gog_tool_move_pie = {
+	N_("Move pie"),
+	GDK_FLEUR,
+	gog_tool_move_pie_point, 
+	gog_tool_move_pie_render,
+	gog_tool_move_pie_init, 
+	gog_tool_move_pie_move, 
+	gog_tool_move_pie_double_click,
+	NULL /*destroy*/
+};
+#endif
+
+/*****************************************************************************/
+
 typedef GogPlotView		GogPieView;
 typedef GogPlotViewClass	GogPieViewClass;
 
+static GogViewClass *pie_view_parent_klass;
+
 #define MAX_ARC_SEGMENTS 64
 
 static void
@@ -416,14 +592,94 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 	else if (num_series > 1)
 		num_series = 1;
 
-	/* centre things */
-	cx = view->allocation.x + view->allocation.w/2.;
-	cy = view->allocation.y + view->allocation.h/2.;
+	/* calculate things when span < 100. */
+	if (model->span < 100.) {
+		double xmin, xmax, ymin, ymax, ratio;
+		double begin, cur, end;
+		if (num_series == 1) {
+			begin = (model->initial_angle + series->initial_angle) * M_PI / 180. - M_PI / 2.;
+			end = begin + model->span * M_PI / 50.;
+		} else {
+			/* WARNING: this code has not been checked */
+			ptr = model->base.series;
+			while (!gog_series_is_valid (GOG_SERIES (ptr->data)))
+				ptr = ptr->next;
+			series = ptr->data;
+			begin = end = series->initial_angle;
+			for (ptr = ptr->next; ptr != NULL; ptr = ptr->next) {
+				if (!gog_series_is_valid (GOG_SERIES (ptr->data)))
+					continue;
+				series = ptr->data;
+				cur = series->initial_angle;
+				if (cur < begin)
+					begin = cur;
+				if (cur > end)
+					end = cur;
+			}
+			begin = (model->initial_angle + begin) * M_PI / 180. - M_PI / 2.;
+			end = ((model->initial_angle + end) / 180. + model->span / 50.) * M_PI - M_PI / 2.;
+		}
+		cur = ceil (begin / M_PI * 2 - 1e-10) * M_PI / 2;
+		xmin = xmax = cos (begin);
+		ymin = ymax = sin (begin);
+		while (cur < end) {
+			cx = cos (cur);
+			cy = sin (cur);
+			if (cx > xmax)
+				xmax = cx;
+			if (cx < xmin)
+				xmin = cx;
+			if (cy > ymax)
+				ymax = cy;
+			if (cy < ymin)
+				ymin = cy;
+			cur += M_PI / 2;
+		}
+		cx = cos (end);
+		cy = sin (end);
+		if (cx > xmax)
+			xmax = cx;
+		if (cx < xmin)
+			xmin = cx;
+		if (cy > ymax)
+			ymax = cy;
+		if (cy < ymin)
+			ymin = cy;
+		/* we ensure that the center will be visible */
+		if (xmin > 0.)
+			xmin = 0.;
+		if (xmax < 0.)
+			xmax = 0.;
+		if (ymin > 0.)
+			ymin = 0.;
+		if (ymax < 0.)
+			ymax = 0.;
+		ratio = (ymax - ymin) / (xmax - xmin);
+		if (view->allocation.h > view->allocation.w * ratio) {
+			r_tot = view->allocation.w * MAX (xmax, -xmin) / (xmax - xmin);
+			cx = view->allocation.x - view->allocation.w * xmin / (xmax - xmin);
+			cy = view->allocation.y +
+				(view->allocation.h + view->allocation.w * ratio) / 2.
+				- view->allocation.w * ratio * ymax / (ymax - ymin);
+		} else {
+			r_tot = view->allocation.h * MAX (ymax, -ymin) / (ymax - ymin);
+			cx = view->allocation.x +
+				(view->allocation.w - view->allocation.h / ratio) / 2.
+				- view->allocation.h / ratio * xmin / (xmax - xmin);
+			cy = view->allocation.y - view->allocation.h * ymin / (ymax - ymin);
+		}
+		r_tot /= 1. + model->default_separation + separation_max;
+	} else {
+		/* centre things */
+		cx = view->allocation.x + view->allocation.w/2.;
+		cy = view->allocation.y + view->allocation.h/2.;
+	
+		r_tot = view->allocation.h;
+		if (r_tot > view->allocation.w)
+			r_tot = view->allocation.w;
+		r_tot /= 2. * (1. + model->default_separation + separation_max);
+	}
 
-	r_tot = view->allocation.h;
-	if (r_tot > view->allocation.w)
-		r_tot = view->allocation.w;
-	r_tot /= 2. * (1. + model->default_separation + separation_max);
 	default_sep = r_tot * model->default_separation;
 	center_radius = r_tot * center_size;
 	r = r_tot * (1. - center_size);
@@ -445,9 +701,9 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 		r_int = center_radius + r * ((double)index - 1.0) / (double)num_series;
 		r_ext = center_radius + r * (double)index / (double)num_series;
 
-		theta = (model->initial_angle + series->initial_angle) * 2. * M_PI / 360. - M_PI / 2.;
+		theta = (model->initial_angle + series->initial_angle) * M_PI / 180. - M_PI / 2.;
 
-		scale = 2 * M_PI / series->total;
+		scale = 2 * M_PI / 100 * model->span / series->total;
 		vals = go_data_vector_get_values (GO_DATA_VECTOR (series->base.values[1].data));
 
 		style = GOG_STYLED_OBJECT (series)->style;
@@ -535,80 +791,22 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 	}
 }
 
-static gboolean
-gog_pie_view_info_at_point (GogView *view, double x, double y,
-			    GogObject const *cur_selection,
-			    GogObject **obj, char **name)
+static void
+gog_pie_view_build_toolkit (GogView *view)
 {
-	GogPiePlot const *model = GOG_PIE_PLOT (view->model);
-	GogPieSeries const *series = NULL;
-	double *vals, scale, len = 0, theta, r = view->allocation.h;
-	GSList *ptr;
-	unsigned i;
-
-	if (r > view->allocation.w)
-		r = view->allocation.w;
-	r /= 2.;
-	x -= view->allocation.x + view->allocation.w/2.;
-	y -= view->allocation.y + view->allocation.h/2.;
-
-	if ((x*x + y*y) > (r*r))
-		return FALSE;
-
-	for (ptr = model->base.series ; ptr != NULL ; ptr = ptr->next)
-		if (gog_series_is_valid (GOG_SERIES (series = ptr->data)))
-			break;
-	if (ptr == NULL)
-		return FALSE;
-	
-	/* TODO what follows does not work for ring plots, so exit here */
-	if (GOG_IS_RING_PLOT (view->model)) {
-		if (obj != NULL)
-			*obj = view->model;
-		if (name != NULL)
-			*name = NULL;
-		return TRUE;
-	}
-
-	theta = (atan2 (y, x) * 180 / M_PI - model->initial_angle + 90.) / 360.;
-	if (theta < 0)
-		theta += 1.;
-
-	vals = go_data_vector_get_values (GO_DATA_VECTOR (series->base.values[1].data));
-	scale = 1 / series->total;
-	for (i = 0 ; i < series->base.num_elements; i++) {
-		len = fabs (vals[i]) * scale;
-		if (go_finite (len) && len > 1e-3) {
-			theta -= len;
-			if (theta < 0)
-				break;
-		}
-	}
-
-	if (obj != NULL) {
-		if (cur_selection == view->model) {
-			*obj = GOG_OBJECT (gog_series_get_element (GOG_SERIES (series), i));
-			if (*obj == NULL) {
-				*obj = g_object_new (gog_pie_series_element_get_type (),
-						     "index", i, NULL);
-				gog_object_add_by_name (GOG_OBJECT (series), "Point", *obj);
-			}
-		} else
-			*obj = view->model;
-	}
-	if (name != NULL)
-		*name = g_strdup_printf (_("%s point %d\nValue %g (%g)"),
-					 gog_object_get_name (GOG_OBJECT (series)),
-					 i+1, vals[i], len);
-
-	return TRUE;
+	(GOG_VIEW_CLASS (pie_view_parent_klass)->build_toolkit) (view);	
+#ifdef WITH_GTK
+	view->toolkit = g_slist_prepend (view->toolkit, &gog_tool_move_pie);
+#endif
 }
 
 static void
 gog_pie_view_class_init (GogViewClass *view_klass)
 {
-	view_klass->render = gog_pie_view_render;
-	view_klass->info_at_point  = gog_pie_view_info_at_point;
+	pie_view_parent_klass = g_type_class_peek_parent (view_klass);
+
+	view_klass->render 		= gog_pie_view_render;
+	view_klass->build_toolkit 	= gog_pie_view_build_toolkit;
 }
 
 GSF_DYNAMIC_CLASS (GogPieView, gog_pie_view,
